# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["RewriteResponse"]


class RewriteResponse(BaseModel):
    rewritten_text: str
    """The rewritten text."""
