# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ConversationSortField"]

ConversationSortField: TypeAlias = Literal["created_at", "updated_at"]
